@@ -49,7 +49,7 @@ class _007::Parser::Actions {
         #      in the expression tree
         if $<EXPR>.ast ~~ Q::Block {
             make Q::Statement::Expr.new(:expr(Q::Postfix::Call.new(
-                :expr($<EXPR>.ast),
+                :operand($<EXPR>.ast),
                 :argumentlist(Q::ArgumentList.new)
             )));
         }
@@ -237,6 +237,11 @@ class _007::Parser::Actions {
             my $infix = @opstack.pop;
             my $t1 = @termstack.pop;
 
+            if $infix ~~ Q::Unquote {
+                @termstack.push(Q::Unquote::Infix.new(:expr($infix.expr), :lhs($t1), :rhs($t2)));
+                return;
+            }
+
             my $c = $*runtime.maybe-get-var($infix.identifier.name.value);
             if $c ~~ Val::Macro {
                 @termstack.push($*runtime.call($c, [$t1, $t2]));
@@ -305,12 +310,18 @@ class _007::Parser::Actions {
 
         sub handle-prefix($/) {
             my $prefix = @prefixes.shift.ast;
+
+            if $prefix ~~ Q::Unquote {
+                make Q::Unquote::Prefix.new(:expr($prefix.expr), :operand($/.ast));
+                return;
+            }
+
             my $c = $*runtime.maybe-get-var($prefix.identifier.name.value);
             if $c ~~ Val::Macro {
                 make $*runtime.call($c, [$/.ast]);
             }
             else {
-                make $prefix.new(:expr($/.ast), :identifier($prefix.identifier));
+                make $prefix.new(:operand($/.ast), :identifier($prefix.identifier));
             }
         }
 
@@ -326,13 +337,13 @@ class _007::Parser::Actions {
                 make $qtree;
             }
             elsif $postfix ~~ Q::Postfix::Index {
-                make $postfix.new(:$identifier, :expr($/.ast), :index($postfix.index));
+                make $postfix.new(:$identifier, :operand($/.ast), :index($postfix.index));
             }
             elsif $postfix ~~ Q::Postfix::Call {
-                make $postfix.new(:$identifier, :expr($/.ast), :argumentlist($postfix.argumentlist));
+                make $postfix.new(:$identifier, :operand($/.ast), :argumentlist($postfix.argumentlist));
             }
             elsif $postfix ~~ Q::Postfix::Property {
-                make $postfix.new(:$identifier, :expr($/.ast), :property($postfix.property));
+                make $postfix.new(:$identifier, :operand($/.ast), :property($postfix.property));
             }
             else {
                 my $c = $*runtime.maybe-get-var($postfix.identifier.name.value);
@@ -340,7 +351,7 @@ class _007::Parser::Actions {
                     make $*runtime.call($c, [$/.ast]);
                 }
                 else {
-                    make $postfix.new(:$identifier, :expr($/.ast));
+                    make $postfix.new(:$identifier, :operand($/.ast));
                 }
             }
         }
@@ -375,7 +386,11 @@ class _007::Parser::Actions {
             :name(Val::Str.new(:value("prefix:<$op>"))),
             :frame($*runtime.current-frame),
         );
-        make $*parser.oplevel.ops<prefix>{$op}.new(:$identifier, :expr(Val::None));
+        make $*parser.oplevel.ops<prefix>{$op}.new(:$identifier, :operand(Val::None));
+    }
+
+    method prefix-unquote($/) {
+        make $<unquote>.ast;
     }
 
     method str($/) {
@@ -499,6 +514,14 @@ class _007::Parser::Actions {
         make $*parser.oplevel.ops<infix>{$op}.new(:$identifier, :lhs(Val::None.new), :rhs(Val::None.new));
     }
 
+    method infix-unquote($/) {
+        my $got = ~($<unquote><identifier> // "Q::Term");
+        die X::TypeCheck.new(:operation<parsing>, :$got, :expected(Q::Infix))
+            unless $got eq "Q::Infix";
+
+        make $<unquote>.ast;
+    }
+
     method postfix($/) {
         my $op = (~$/).trim;
         if $<index> {  # XXX: more hardcoding :(
@@ -517,16 +540,16 @@ class _007::Parser::Actions {
         # XXX: this can't stay hardcoded forever, but we don't have the machinery yet
         # to do these right enough
         if $<index> {
-            make Q::Postfix::Index.new(index => $<EXPR>.ast, :$identifier, :expr(Val::None.new));
+            make Q::Postfix::Index.new(index => $<EXPR>.ast, :$identifier, :operand(Val::None.new));
         }
         elsif $<call> {
-            make Q::Postfix::Call.new(argumentlist => $<argumentlist>.ast, :$identifier, :expr(Val::None.new));
+            make Q::Postfix::Call.new(argumentlist => $<argumentlist>.ast, :$identifier, :operand(Val::None.new));
         }
         elsif $<prop> {
-            make Q::Postfix::Property.new(property => $<identifier>.ast, :$identifier, :expr(Val::None.new));
+            make Q::Postfix::Property.new(property => $<identifier>.ast, :$identifier, :operand(Val::None.new));
         }
         else {
-            make $*parser.oplevel.ops<postfix>{$op}.new(:$identifier, :expr(Val::None.new));
+            make $*parser.oplevel.ops<postfix>{$op}.new(:$identifier, :operand(Val::None.new));
         }
     }
 
