@@ -144,9 +144,9 @@ grammar _007::Parser::Syntax {
         || <.ws> $
     }
 
-    rule EXPR { <termish> +% <infix> }
+    rule EXPR { <termish> +% [<infix> | <infix=infix-unquote>] }
 
-    token termish { <prefix>* [<term>|<term=unquote>] <postfix>* }
+    token termish { [<prefix> | <prefix=prefix-unquote>]* [<term>|<term=unquote>] <postfix>* }
 
     method prefix {
         my @ops = $*parser.oplevel.ops<prefix>.keys;
@@ -219,7 +219,7 @@ grammar _007::Parser::Syntax {
 
     token propertylist { [<.ws> <property>]* % [\h* ','] <.ws> }
 
-    token unquote { '{{{' <EXPR> '}}}' }
+    token unquote { '{{{' <EXPR> [:s "@" <identifier> ]? '}}}' }
 
     proto token property {*}
     rule property:str-expr { <key=str> ':' <value=term> }
@@ -242,6 +242,14 @@ grammar _007::Parser::Syntax {
         return /<!>/(self);
     }
 
+    rule infix-unquote {
+        <unquote>
+    }
+
+    rule prefix-unquote {
+        <unquote> <?{ ($<unquote><identifier> // "") eq "Q::Prefix" }>
+    }
+
     method postfix {
         # XXX: should find a way not to special-case [] and () and .
         if /$<index>=[ <.ws> '[' ~ ']' [<.ws> <EXPR>] ]/(self) -> $cur {
