@@ -156,12 +156,12 @@ class Q::Block does Q {
 
 class Q::Prefix does Q::Expr {
     has $.identifier;
-    has $.expr;
+    has $.operand;
 
-    method attribute-order { <identifier expr> }
+    method attribute-order { <identifier operand> }
 
     method eval($runtime) {
-        my $e = $.expr.eval($runtime);
+        my $e = $.operand.eval($runtime);
         my $c = $.identifier.eval($runtime);
         return $runtime.call($c, [$e]);
     }
@@ -169,6 +169,8 @@ class Q::Prefix does Q::Expr {
 
 class Q::Prefix::Minus is Q::Prefix {}
 
+class Q::Prefix::Not is Q::Prefix {}
+
 class Q::Infix does Q::Expr {
     has $.identifier;
     has $.lhs;
@@ -238,16 +240,14 @@ class Q::Infix::And is Q::Infix {
     }
 }
 
-class Q::Prefix::Not is Q::Prefix {}
-
 class Q::Postfix does Q::Expr {
     has $.identifier;
-    has $.expr;
+    has $.operand;
 
-    method attribute-order { <identifier expr> }
+    method attribute-order { <identifier operand> }
 
     method eval($runtime) {
-        my $e = $.expr.eval($runtime);
+        my $e = $.operand.eval($runtime);
         my $c = $.identifier.eval($runtime);
         return $runtime.call($c, [$e]);
     }
@@ -256,10 +256,10 @@ class Q::Postfix does Q::Expr {
 class Q::Postfix::Index is Q::Postfix {
     has $.index;
 
-    method attribute-order { <identifier expr index> }
+    method attribute-order { <identifier operand index> }
 
     method eval($runtime) {
-        given $.expr.eval($runtime) {
+        given $.operand.eval($runtime) {
             when Val::Array {
                 my $index = $.index.eval($runtime);
                 die X::Subscript::NonInteger.new
@@ -285,10 +285,10 @@ class Q::Postfix::Index is Q::Postfix {
 class Q::Postfix::Call is Q::Postfix {
     has $.argumentlist;
 
-    method attribute-order { <identifier expr argumentlist> }
+    method attribute-order { <identifier operand argumentlist> }
 
     method eval($runtime) {
-        my $c = $.expr.eval($runtime);
+        my $c = $.operand.eval($runtime);
         die "macro is called at runtime"
             if $c ~~ Val::Macro;
         die "Trying to invoke a {$c.^name.subst(/^'Val::'/, '')}" # XXX: make this into an X::
@@ -301,10 +301,10 @@ class Q::Postfix::Call is Q::Postfix {
 class Q::Postfix::Property is Q::Postfix {
     has $.property;
 
-    method attribute-order { <identifier expr property> }
+    method attribute-order { <identifier operand property> }
 
     method eval($runtime) {
-        my $obj = $.expr.eval($runtime);
+        my $obj = $.operand.eval($runtime);
         my $propname = $.property.name.value;
         $runtime.property($obj, $propname);
     }
@@ -318,6 +318,15 @@ class Q::Unquote does Q {
     }
 }
 
+class Q::Unquote::Prefix is Q::Unquote {
+    has $.operand;
+}
+
+class Q::Unquote::Infix is Q::Unquote {
+    has $.lhs;
+    has $.rhs;
+}
+
 class Q::Term::Quasi does Q::Term {
     has $.contents;
 
@@ -335,6 +344,19 @@ class Q::Term::Quasi does Q::Term {
             return $thing.new(:name($thing.name), :frame($runtime.current-frame))
                 if $thing ~~ Q::Identifier;
 
+            if $thing ~~ Q::Unquote::Prefix {
+                my $prefix = $thing.expr.eval($runtime);
+                die X::TypeCheck.new(:operation("interpolating an unquote"), :got($prefix), :expected(Q::Prefix))
+                    unless $prefix ~~ Q::Prefix;
+                return $prefix.new(:identifier($prefix.identifier), :operand($thing.operand));
+            }
+            elsif $thing ~~ Q::Unquote::Infix {
+                my $infix = $thing.expr.eval($runtime);
+                die X::TypeCheck.new(:operation("interpolating an unquote"), :got($infix), :expected(Q::Infix))
+                    unless $infix ~~ Q::Infix;
+                return $infix.new(:identifier($infix.identifier), :lhs($thing.lhs), :rhs($thing.rhs));
+            }
+
             if $thing ~~ Q::Unquote {
                 my $ast = $thing.expr.eval($runtime);
                 die "Expression inside unquote did not evaluate to a Q" # XXX: turn into X::
@@ -383,7 +405,7 @@ class Q::Statement::My does Q::Statement does Q::Declaration {
     has $.identifier;
     has $.expr = Val::None.new;
 
-    method attribute-order { <expr identifier> }
+    method attribute-order { <identifier expr> }
 
     method is-assignable { True }
 
@@ -399,7 +421,7 @@ class Q::Statement::Constant does Q::Statement does Q::Declaration {
     has $.identifier;
     has $.expr;
 
-    method attribute-order { <expr identifier> }
+    method attribute-order { <identifier expr> }
 
     method run($runtime) {
         # value has already been assigned
