@@ -402,6 +402,8 @@ class _007::Runtime::Builtins {
             Q::Trait,
             Q::TraitList,
             Q::Unquote,
+            Q::Unquote::Infix,
+            Q::Unquote::Prefix,
         ;
 
         for @q-types -> $type {
